﻿using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Web;
using SaveAsOpenDemo.Models;

namespace SaveAsOpenDemo.Common
{
    public class DataService
    {
        private static string _fqFilePath;
        private static string _fileName;
        private static string _filePath;

        private static bool _pathSet;
        public string Connection;
        
        public DataService(string path, string filename)
        {
            _pathSet = true;
            _filePath = path;
            _fileName = filename;
            _fqFilePath = Path.Combine(path, filename);

            EnsureDirectory(_fqFilePath);
        }

        public DataService(string fqPath)
        {
            try
            {
                _pathSet = true;

                _filePath = Path.GetDirectoryName(fqPath);
                _fileName = Path.GetFileName(fqPath);

                _fqFilePath = fqPath;
                EnsureDirectory(_fqFilePath);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
                throw new ArgumentOutOfRangeException();
            }
        }

        private static void SetFilePath()
        {
            if (!string.IsNullOrWhiteSpace(_fqFilePath))
            {
                _fqFilePath = Path.Combine(AssemblyDirectory, "mockDb.txt");
                EnsureDirectory(_fqFilePath);
                _pathSet = true;
            }
        }

        private static void WriteToFile(string message)
        {
            if (!_pathSet) SetFilePath();

            using (StreamWriter writer = new StreamWriter(_fqFilePath, true))
            {
                writer.WriteLine(message);
                writer.Close();
            }
        }

        private static IList<SReport> ReadFileAll(string fileName)
        {
            if (!_pathSet) SetFilePath();
            
            var retList = new List<SReport>();

            using (StreamReader reader = new StreamReader(_fqFilePath))
            {
                while (!reader.EndOfStream)
                {
                    var dataLine = reader.ReadLine();
                    retList.Add(new SReport{FileName = dataLine});
                }
                reader.Close();
            }

            return retList;
        }

        private static void EnsureDirectory(string filePath)
        {
            FileInfo fi = new FileInfo(filePath);

            if (fi.Directory != null && !fi.Directory.Exists)
                if (fi.DirectoryName != null) 
                    Directory.CreateDirectory(fi.DirectoryName);

        }

        public static string AssemblyDirectory
        {
            get
            {
                string codeBase = Assembly.GetExecutingAssembly().CodeBase; 
                UriBuilder uri = new UriBuilder(codeBase); 
                string path = Uri.UnescapeDataString(uri.Path); 
                return Path.GetDirectoryName(path);
            }
        }

        internal IList<SReport> GetList()
        {
            return ReadFileAll(_fqFilePath);
        }

        internal void Save(SReport srReport)
        {
            WriteToFile(srReport.FileName);
        }
    }
}