﻿using Newtonsoft.Json;
using SaveAsOpenDemo.Common;
using SaveAsOpenDemo.Models;
using Stimulsoft.Report;
using Stimulsoft.Report.MobileDesign;
using Stimulsoft.Report.MvcMobile;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Diagnostics;
using System.Linq;
using System.Web;
using System.Web.Hosting;
using System.Web.Mvc;
using SaveAsOpenDemo.Models.Utility;

namespace SaveAsOpenDemo.Controllers
{
    public class SRDesignerController : Controller
    {
        // Any pathing variable beginging with 'fq' denotes a 'Fully Qualified' filename and path

        private DataService _repo;

        public SRDesignerController()
        {
            var fqFile = HostingEnvironment.MapPath("~/App_UserData/mockDb.txt");
            _repo = new DataService(fqFile);
        }

        public ActionResult Index()
        {
            return View();
        }

        public PartialViewResult GetReportListModal()
        {
            var reports = _repo.GetList();
            return PartialView("_ReportList", reports);
        }

        public ActionResult GetReportTemplate(string templateFileName = "")
        {
            StiReport report = Session["GetReportTemplate"] as StiReport ?? new StiReport();

            //StiReport report = new StiReport();
            templateFileName = !string.IsNullOrWhiteSpace(templateFileName) ? templateFileName : "TemplateBase";
            templateFileName = templateFileName.Replace(".mrt", "");

            var reportLocation = templateFileName == "TemplateBase"
                                               ? string.Format("/Views/SRDesigner/{0}.mrt", templateFileName)
                                               : string.Format("/App_UserData/UserReports/{0}.mrt", templateFileName);

            var fqReportFile = HostingEnvironment.MapPath(reportLocation);

            try
            {
                report.Load(fqReportFile);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }

            return StiMvcMobileDesigner.GetReportTemplateResult(HttpContext, report);
        }

        public ActionResult GetReportSnapshot()
        {
            StiReport report = StiMvcMobileDesigner.GetReportObject(HttpContext);

            DataSet data = new DataSet("Demo");
            var fqReportFile = HostingEnvironment.MapPath(string.Format("/Views/SRDesigner/Data/{0}.xml", "Demo.xml"));

            try
            {
                data.ReadXml(fileName: fqReportFile);
                report.RegData(data);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }

            return StiMvcMobileDesigner.GetReportSnapshotResult(HttpContext);
        }

        [HttpPost]
        public ActionResult SaveAsReportTemplate(object mvcMobileDesignerParameters)
        {
            StiReport report = StiMvcMobileDesigner.GetReportObject(HttpContext);
            Hashtable rptParameters = StiMvcMobileDesigner.GetHttpContextParameters(HttpContext);

            // N2L Probably the same
            var rptFile = rptParameters["reportFile"].ToString();
            string fqReportFile = GetFqReportPath(rptFile);

            var srReport = FillSReportSaveInfo(rptFile, fqReportFile);

            // Mock save of report 'info' to database
            _repo.Save(srReport);

            try
            {
                // Sti-Report Save Attempt
                report.Save(fqReportFile);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }

            return StiMvcMobileDesigner.SaveReportTemplateResult(HttpContext);
        }

        [HttpPost]
        public ActionResult OpenSavedReportTemplate(SsrdOpenFileParameters mvcMobileDesignerParameters) 
        {
            var rptData = mvcMobileDesignerParameters;

            StiReport report = new StiReport();
            var fqFile = (rptData != null) ? rptData.reportFile : "";
            
            string fqReportFile = GetFqReportPath(fqFile);

            try
            {
                report.Load(fqReportFile);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }

            // :: LOOK HERE :: LOOK HERE :: LOOK HERE :: LOOK HERE :: 
            
            // What to do here? 
            return View("Index", report);

        }

        /// <summary>
        /// Get fully qualified path and file name
        /// </summary>
        /// <param name="templateFileName"></param>
        /// <returns></returns>
        private string GetFqReportPath(string templateFileName = null)
        {
            templateFileName = !string.IsNullOrWhiteSpace(templateFileName) ? templateFileName : "TemplateBase";
            templateFileName = templateFileName.Replace(".mrt", "");

            var reportLocation = templateFileName == "TemplateBase"
                                               ? string.Format("/Views/SRDesigner/{0}.mrt", templateFileName)
                                               : string.Format("/App_UserData/UserReports/{0}.mrt", templateFileName);

            var fqFile = HostingEnvironment.MapPath(reportLocation);

            return fqFile;
        }

        private SReport FillSReportSaveInfo(string rptName, string fqFileName)
        {
            return new SReport
            {
                FileName = rptName,
                FullyQualifiedFile = fqFileName
            };
        }

        public ActionResult DesignerEvent(object mvcMobileDesignerParameters = null)
        {
            return StiMvcMobileDesigner.DesignerEventResult(HttpContext);
        }
    }
}